#!/usr/bin/env python
#coding: utf-8


import milena,subprocess,pyspda,time,threading,SocketServer,socket,signal,sapilektor

class milena_command(object):
    def __init__(self,text,sync,freq=16000,rate=0,pitch=0):
        self.text=text
        self.freq=freq
        self.rate=rate
        self.pitch=pitch
        if not sync:
            self.syncvar=None
        else:
            self.syncvar=threading.Condition()
            self.syncvar.acquire()

class MyTCPHandler(SocketServer.StreamRequestHandler):

    def handle(self):
        self.data = self.request.recv(8192).strip()
        if self.data == '?':
            rc=self.server.audioplayer.is_playing()
        else:    
            if self.data == '':
                txt=None
                sync=False
            elif self.data.startswith('+'):
                txt=self.data[1:]
                if not txt:
                    txt=None
                    sync=False
                else:
                    sync=True
            else:
                txt=self.data
                sync=False
            rc=self.server.audioplayer.send_command(txt,sync)
        if rc:
            rc='t\n'
        else:
            rc='f\n'
        self.wfile.write(rc)
        # Likewise, self.wfile is a file-like object used to write back
        # to the client
        #self.wfile.write(self.data.upper())

class MyTCPServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
    allow_reuse_address = True
    pass

class AudioTCPServer(MyTCPServer):
    def __init__(self,*args):
        MyTCPServer.__init__(self,*args)
        self.audioplayer=AudioPlayer()
        
    def shutdown(self):
        a=self.audioplayer
        self.audioplayer=None
        a.__del__()
        MyTCPServer.shutdown(self)
        


class AudioPlayer(object):
    def __init__(self):
        self.Milena=milena.Milena()
        self.Ivona=milena.Milena(ivoice='Ewa')
        try:
            self.Lektor=sapilektor.Lektor()
        except:
            self.Lektor=None
        self.spda=pyspda.Spda(3)
        self.spda.contrast(80)
        self.spda.vol(100)
        self.spda.equalizer(0.5,1.0,1.0);
        self.lock=threading.Lock()
        self.register=None
        self.quito=False
        self.player=threading.Thread(target=self.do_play)
        self.player.start()
        self._playing=False
    

    def __del__(self):
        self.quito=True
        self.spda.stop()
        self.player.join()
        
    def phonemize(self,text):
        text=text.split('\n')
        self.Milena.feed()
        sout=''
        for row in text:
            self.Milena.feed(row)
            while True:
                s=self.Milena.consume()
                if not s:
                    break
                sout += s[0]
        return sout

    def ivonize(self,text):
        text=text.split('\n')
        self.Ivona.feed()
        sout=[]
        for row in text:
            self.Ivona.feed(row)
            while True:
                s=self.Ivona.consume()
                if not s:
                    break
                sout.append(s[0])
        return ' '.join(sout)

                    
    def mbroler(self,sout):
        p=subprocess.Popen(['mbrola','-e','-f','0.9','-t','0.9','/usr/share/mbrola/pl1/pl1','-','-'],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        try:
            rc=p.communicate(sout)
            return rc[0]
        except:
            return None


    def is_playing(self):
        return self._playing
        
    def get_command(self):
        while not self.quito:
            self.lock.acquire()
            cmd=self.register
            self.register=None
            if cmd:
                self.spda.enable()
                self._playing=True
            elif self._playing:
                self._playing=False
            self.lock.release()
            if cmd:
                return cmd
            time.sleep(0.1)
        return None
        
    def send_command(self,text=None,sync=False):
        if not text:
            sync=False
        self.lock.acquire()
        if self.register:
            if self.register.syncvar:
                self.register.syncvar.broken=1
                self.register.syncvar.acquire()
                self.register.syncvar.notify()
                self.register.syncvar.release()
            self.register=None
        if text:
            cmd=milena_command(text,sync)
            self.register=cmd
        self.spda.stop()
        self.lock.release()
        if not sync:
            return True
        cmd.syncvar.wait()
        return cmd.syncvar.broken == 0
    
    def do_play(self):
        while not self.quito:
            cmd=self.get_command() # locked, spda.enable 
            if not cmd:             # np. stop
                continue
            wave=None
            freq=16000
            rate=cmd.rate
            if self.Lektor:
                try:
                    wave,freq=self.Lektor.GetWave(self.ivonize(cmd.text),1)
                    rate=20
                    self.spda.contrast(0)
                    self.spda.vol(60)
                except:
                    pass
            if not wave:
                phonemes=self.phonemize(cmd.text)
                if not self.spda.stopped:
                    wave=self.mbroler(phonemes)
                    freq=16000
                    self.spda.contrast(80)
                    self.spda.vol(100)
            if not self.spda.stopped:
                self.spda.play(wave,freq=freq,rate=rate,pitch=cmd.pitch)
                self.spda.end()
            if cmd.syncvar:
                cmd.syncvar.broken=self.spda.stopped
                cmd.syncvar.acquire()
                cmd.syncvar.notify()
                cmd.syncvar.release()


server = AudioTCPServer(('localhost', 6701), MyTCPHandler)
signal.signal(signal.SIGTERM,signal.SIG_DFL)
try:
    server.serve_forever()
except:
    server.shutdown()
    
